//+------------------------------------------------------------------+
//|                                                         iSCH.mq4 |
//|                                                Sergei Vladimirov |
//|          https://www.mql5.com/ru/users/sergey_mechanic/portfolio |
//+------------------------------------------------------------------+
#property copyright "Sergei Vladimirov"
#property link      "https://www.mql5.com/ru/users/sergey_mechanic/portfolio"
#property version   "2.00"
#property strict
#property indicator_chart_window
#property indicator_buffers 4

#property indicator_color1 clrAqua
#property indicator_color2 clrDeepPink
#property indicator_color3 clrYellow
#property indicator_color4 clrYellow

#property indicator_width1 2
#property indicator_width2 2

#define TIME_ERR_VAL 86401
#define TIME_EMPTY_VAL 86400

// inputs
sinput string					i_sStartTime		= "00:00";					//   
sinput string					i_sEndTime			= "09:00";					//   

// buffers
double BuyBuffer[];
double SellBuffer[];
double UpperBuffer[];
double LowerBuffer[];

// globals
datetime g_dtStartTime;
datetime g_dtEndTime;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
{
	// input parameters parsing
	g_dtStartTime = ParseTimeString(i_sStartTime);
	g_dtEndTime = ParseTimeString(i_sEndTime);
	
	// check for errors
	string sErr = "";
	
	if(g_dtStartTime == TIME_ERR_VAL || g_dtStartTime == TIME_EMPTY_VAL)
	{
		if(TerminalInfoString(TERMINAL_LANGUAGE) == "Russian")
			sErr = "     \n";
		else
			sErr = "Incorrect Start Time value";
	}
	
	if(g_dtEndTime == TIME_ERR_VAL || g_dtEndTime == TIME_EMPTY_VAL)
	{
		if(TerminalInfoString(TERMINAL_LANGUAGE) == "Russian")
			sErr = "    ";
		else
			sErr = "Incorrect End Time value";
	}
	
	if(sErr != "")
	{
		Alert(sErr);
		return(INIT_FAILED);
	}
	
	// buffers
	SetIndexBuffer(0, BuyBuffer);
	SetIndexStyle(0, DRAW_ARROW);
	SetIndexArrow(0, 159);
	SetIndexEmptyValue(0, 0.0);
	SetIndexLabel(0, "Buy");
	
	SetIndexBuffer(1, SellBuffer);
	SetIndexStyle(1, DRAW_ARROW);
	SetIndexArrow(1, 159);
	SetIndexEmptyValue(1, 0.0);
	SetIndexLabel(1, "Sell");
	
	SetIndexBuffer(2, UpperBuffer);
	SetIndexStyle(2, DRAW_LINE);
	SetIndexEmptyValue(2, 0.0);
	SetIndexLabel(2, "Upper Level");
	
	SetIndexBuffer(3, LowerBuffer);
	SetIndexStyle(3, DRAW_LINE);
	SetIndexEmptyValue(3, 0.0);
	SetIndexLabel(3, "Lower Level");
	
	IndicatorDigits(Digits());
	
   return(INIT_SUCCEEDED);
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
{
	static double s_fHH = 0;
	static double s_fLL = 0;
	
	static bool s_bStartDrawing = false;
	static bool s_bSignalIsAllowed = false;

	if(prev_calculated == 0)
	{
		ArrayInitialize(BuyBuffer, 0.0);
		ArrayInitialize(SellBuffer, 0.0);
		ArrayInitialize(UpperBuffer, 0.0);
		ArrayInitialize(LowerBuffer, 0.0);
		
		s_fHH = high[rates_total-1];
		s_fLL = low[rates_total-1];
		
		s_bStartDrawing = false;
		s_bSignalIsAllowed = false;
	}
	
	int nStartBar = rates_total - MathMax(prev_calculated, 2);
	
	for(int i = nStartBar; i >= 0; i--)
	{
		//     
		if(IsInTimeRange(time[i], g_dtStartTime, g_dtEndTime) == true)
		{
			UpperBuffer[i] = UpperBuffer[i+1];
			LowerBuffer[i] = LowerBuffer[i+1];
		
			//   
			if(IsInTimeRange(time[i+1], g_dtStartTime, g_dtEndTime) == false)
			{
				s_fHH = high[i];
				s_fLL = low[i];
			}
			//   
			else
			{
				s_fHH = MathMax(s_fHH, high[i]);
				s_fLL = MathMin(s_fLL, low[i]);
			}
			
			s_bStartDrawing = true;
			s_bSignalIsAllowed = true;
		}
		else if(s_bStartDrawing == true)
		{
			UpperBuffer[i] = s_fHH;
			LowerBuffer[i] = s_fLL;
			
			// only 1 signal per day is allowed
			if(s_bSignalIsAllowed == true)
			{
				if(high[i] > UpperBuffer[i])
				{
					BuyBuffer[i] = UpperBuffer[i];
					s_bSignalIsAllowed = false;
				}
				
				if(low[i] < LowerBuffer[i])
				{
					SellBuffer[i] = LowerBuffer[i];
					s_bSignalIsAllowed = false;
				}
			}
		}
	}
	
   return(rates_total);
}

//+------------------------------------------------------------------+

// convert intput string HH:MM or HH:MM:SS into seconds from midnight
datetime ParseTimeString(const string sTimeStr)
{
	// error and empty values
	datetime dtErrVal = 86400 + 1;
	datetime dtEmptyVal = 86400;

	// delete spaces from input string
	string sInputString = sTimeStr;
	StringReplace(sInputString, " ", "");
	
	// empty string
	if(sInputString == "")
		return(dtEmptyVal);
	
	// check input string characters: only digits and separator ':' are allowed
	int nLength = StringLen(sInputString);
	char sChar = 0;
	for(int i = 0; i < nLength; i++)
	{
		sChar = (char) StringGetCharacter(sTimeStr, i);
		
		if((sChar < '0' || sChar > '9') && sChar != ':')
			return (dtErrVal);
	}
	
	// array for parsing
	string sSubStrings[];
	ZeroMemory(sSubStrings);
	
	// parse input line
	int nSubStrings = StringSplit(sInputString, ':', sSubStrings);
	
	// format error
	if(nSubStrings < 2 || nSubStrings > 3)
		return(dtErrVal);
		
	int nHour = (int) StringToInteger(sSubStrings[0]);
	int nMinute = (int) StringToInteger(sSubStrings[1]);
	
	int nSeconds = 0;
	
	if(nSubStrings == 3)
		nSeconds = (int) StringToInteger(sSubStrings[2]);
	
	if(nHour < 0 || nHour > 23 || nMinute < 0 || nMinute > 59 || nSeconds < 0 || nSeconds > 59)
		return(dtErrVal);
	
	return( (datetime) (nHour * 3600 + nMinute * 60 + nSeconds) );
}

//+------------------------------------------------------------------+

bool IsInTimeRange(const datetime dtTime, const datetime dtStartTime, const datetime dtEndTime)
{
	//    ,     ,   true
	if(dtStartTime == TIME_EMPTY_VAL || dtEndTime == TIME_EMPTY_VAL)
		return(true);
	
	if(dtStartTime == TIME_ERR_VAL || dtEndTime == TIME_ERR_VAL)
		return(true);
	
	
	bool bRetVal = false;
	
	datetime dtIntradayTime = dtTime % 86400;
	
	if(dtEndTime > dtStartTime)
		bRetVal = dtIntradayTime >= dtStartTime && dtIntradayTime < dtEndTime;

	if(dtEndTime < dtStartTime)
		bRetVal = dtIntradayTime >= dtStartTime || dtIntradayTime < dtEndTime;

	
	return(bRetVal);
}

//+------------------------------------------------------------------+
